/**
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0,
 * and you may not use this file except in compliance with the Apache License Version 2.0.
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Apache License Version 2.0 is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.guice.bean.binders;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.inject.Inject;

import org.sonatype.guice.bean.locators.BeanLocator;
import org.sonatype.guice.bean.locators.EntryListAdapter;
import org.sonatype.guice.bean.locators.EntryMapAdapter;
import org.sonatype.guice.bean.locators.NamedIterableAdapter;

import com.google.inject.Key;
import com.google.inject.Provider;
import com.google.inject.name.Named;

// ----------------------------------------------------------------------
// BeanLocator-backed Providers that can provide dynamic bean lookups
// ----------------------------------------------------------------------

/**
 * Provides a {@link List} of qualified beans.
 */
@SuppressWarnings( "unchecked" )
final class BeanListProvider<K extends Annotation, V>
    implements Provider<List<V>>
{
    @Inject
    private BeanLocator locator;

    private final Key<V> key;

    BeanListProvider( final Key<V> key )
    {
        this.key = key;
    }

    public List<V> get()
    {
        return new EntryListAdapter( locator.locate( key ) );
    }
}

// ----------------------------------------------------------------------

/**
 * Provides a {@link Map} of qualified beans.
 */
@SuppressWarnings( "unchecked" )
final class BeanMapProvider<K extends Annotation, V>
    implements Provider<Map<K, V>>
{
    @Inject
    private BeanLocator locator;

    private final Key<V> key;

    BeanMapProvider( final Key<V> key )
    {
        this.key = key;
    }

    public Map<K, V> get()
    {
        return new EntryMapAdapter( locator.locate( key ) );
    }
}

// ----------------------------------------------------------------------

/**
 * Provides a {@link Map} of named beans.
 */
@SuppressWarnings( "unchecked" )
final class NamedBeanMapProvider<V>
    implements Provider<Map<String, V>>
{
    @Inject
    private BeanLocator locator;

    private final Key key;

    NamedBeanMapProvider( final Type valueType )
    {
        this.key = Key.get( valueType, Named.class );
    }

    public Map<String, V> get()
    {
        return new EntryMapAdapter( new NamedIterableAdapter( locator.locate( key ) ) );
    }
}

// ----------------------------------------------------------------------

/**
 * Provides a single qualified bean.
 */
final class BeanProvider<V>
    implements Provider<V>
{
    @Inject
    private BeanLocator locator;

    private final Key<V> key;

    BeanProvider( final Key<V> key )
    {
        this.key = key;
    }

    public V get()
    {
        final Iterator<Entry<Annotation, V>> i = locator.locate( key ).iterator();
        return i.hasNext() ? i.next().getValue() : null; // TODO: dynamic proxy??
    }
}

// ----------------------------------------------------------------------
